//@author:Noir
//@help: 
//@tags: phong,wiggle,fog
//@credits: vux,dottore,robe



#include "PhongDirectional.fxh"

// -----------------------------------------------------------------------------
// PARAMETERS:
// -----------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        
float4x4 tV: VIEW;         
float4x4 tWV: WORLDVIEW;
float4x4 tVP : VIEWPROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION; 
float4x4 tWIT: WORLDINVERSETRANSPOSE;


Texture2D tex0 <string uiname="DiffuseTexture"; >; 
SamplerState g_samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

float4x4 tTex <bool uvspace=true; string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

//Wiggle
float time=1.0;
float Freq = 1.0;
float Ampl = 1.0;
float timeScale = 4.0;

//Fog
	float4 fogcolor <bool color=true;String uiname="FogColor";> = { 0.5f, 0.5f, 0.5f,1.0f };
    float3 CameraPosition;
//	float fogStart;
//  float fogEnd;
    float FogDensity;
    float ExponetialFactor = 2.71828 ;    

struct vs2ps
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd : TEXCOORD0;
    float3 LightDirV: TEXCOORD1;
    float3 NormV: TEXCOORD2;
    float3 ViewDirV: TEXCOORD3;
	float fogFactor : FOG;
};

// -----------------------------------------------------------------------------
// VERTEXSHADERS
// -----------------------------------------------------------------------------

vs2ps VS(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
	
	float4 cameraPosition;
	
	PosO.x+=sin(PosO.y* Freq+ time*timeScale) * Ampl; //Original
	Out.PosWVP  = mul(PosO,mul(tW,tVP));
   
    //inverse light direction in view space
    Out.LightDirV = normalize(-mul(float4(lDir,0.0f), mul(tW,tV)).xyz);
	
    //normal in view space
    Out.NormV = normalize(mul(mul(NormO, (float3x3)tWIT),(float3x3)tV).xyz);

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    Out.ViewDirV = -normalize(mul(PosO, tWV)).xyz;
	
	cameraPosition = mul(PosO, tW);
    cameraPosition = mul(cameraPosition, tV);

	//Exponential Fog 
	//Out.fogFactor=  pow((1.0 / abs(ExponetialFactor)),(cameraPosition.z * FogDensity));
	
    //Exponential Fog 2
  	Out.fogFactor =   pow((1.0 /abs(ExponetialFactor)),(((cameraPosition.z * FogDensity) * (cameraPosition.z * FogDensity))));
	
	
    return Out;
}

// -----------------------------------------------------------------------------
// PIXELSHADERS:
// -----------------------------------------------------------------------------

float Alpha <float uimin=0.0; float uimax=1.0;> = 1;

float4 PS(vs2ps In): SV_Target
{
   
    float4 fogColor;
   	fogColor =  fogcolor ;
	
	float4 col = tex0.Sample(g_samLinear, In.TexCd.xy);
   col *= PhongDirectional(In.NormV, In.ViewDirV, In.LightDirV)+ (1.0 - In.fogFactor) * fogColor;
	//col.rgb *= PhongDirectional(In.NormV, In.ViewDirV, In.LightDirV)+ (1.0 - In.fogFactor) * fogColor;
	
    col.a *= Alpha;
   	return mul(col, tColor);
	
}


// -----------------------------------------------------------------------------
// TECHNIQUES:
// -----------------------------------------------------------------------------
technique11 MrWiggleTexturedFOG
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
